# -*- coding: utf-8 -*-
from typing import List
import logging
from sqlalchemy import Column, Integer, String, DateTime, func, text
from sqlalchemy.ext.declarative import declarative_base

from website.db_mysql import get_session, to_json_list

"""
CREATE TABLE `enterprise_busi_model_node_device` (
  `id` int NOT NULL,
  `suid` varchar(10) DEFAULT NULL,
  `entity_suid` varchar(10) DEFAULT NULL,
  `node_id` int DEFAULT NULL,
  `node_suid` varchar(10) DEFAULT NULL,
  `busi_model_id` int DEFAULT NULL,
  `busi_model_suid` varchar(10) DEFAULT NULL,
  `device_id` int DEFAULT NULL,
  `device_suid` varchar(10) DEFAULT NULL,
  `create_time` datetime DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='节点-业务模型-设备关联表';
"""

Base = declarative_base()


class EnterpriseBusiModelNodeDevice(Base):
    __tablename__ = "enterprise_busi_model_node_device"

    id = Column(Integer, primary_key=True)
    suid = Column(String(10))
    entity_suid = Column(String(10))
    node_id = Column(Integer)
    node_suid = Column(String(10))
    busi_model_id = Column(Integer)
    busi_model_suid = Column(String(10))
    device_id = Column(Integer)
    device_suid = Column(String(10))
    create_time = Column(
        DateTime, default=func.current_timestamp(), onupdate=func.current_timestamp()
    )

    def __repr__(self):
        return f"EnterpriseBusiModelNodeDevice(id={self.id}, suid='{self.suid}')"


class EnterpriseBusiModelNodeDeviceRepository(object):

    def insert_record(self, records: List[EnterpriseBusiModelNodeDevice]):
        with get_session() as session:
            for record in records:
                session.add(record)
            session.commit()

    def batch_insert_record(self, records: List[dict]):
        with get_session() as session:
            session.bulk_insert_mappings(EnterpriseBusiModelNodeDevice, records)
            session.commit()

    def delete_by_busi_model_and_node(self, node_id: int, busi_model_id: int) -> bool:
        try:
            with get_session() as session:
                sql = text(
                    "delete from enterprise_busi_model_node_device where node_id=:node_id and busi_model_id=:busi_model_id"
                )
                session.execute(
                    sql, {"node_id": node_id, "busi_model_id": busi_model_id}
                )
                session.commit()
        except Exception as e:
            logging.info("delete_by_busi_model_and_node error: %s", e)
            return False

        return True

    def check_deployed(self, node_id: int, busi_model_id: int) -> int:
        with get_session() as session:
            count = (
                session.query(EnterpriseBusiModelNodeDevice)
                .filter(
                    EnterpriseBusiModelNodeDevice.node_id == node_id,
                    EnterpriseBusiModelNodeDevice.busi_model_id == busi_model_id,
                )
                .count()
            )
            return 1 if count > 0 else 0

    def get_device_ids(self, node_id: int, busi_model_id: int) -> list:
        with get_session() as session:
            sql = text(
                "select device_id from enterprise_busi_model_node_device where node_id=:node_id and busi_model_id=:busi_model_id"
            )
            res = session.execute(
                sql, {"node_id": node_id, "busi_model_id": busi_model_id}
            )
            return to_json_list(res)
